/*
Adds A dialog for managing dopus folder aliases.

Sample button to open dialog.
*/

// -------------------- Options
var gbUseLogging=true;

// Called by Directory Opus to initialize the script.
function OnInit(initData){
	var uid = "306f852a-e136-4829-913b-5256fa2233a4";
	var url = "http://resource.dopus.com/";
	initData.url = url;
	initData.name = "Manage Dopus folder alaises";
	initData.desc = "Dialog for viewing and managing Dopus folder alaises.";
	initData.copyright = "wowbagger";
	initData.version = "0.2";
	initData.default_enable = true;
	initData.config_desc = DOpus.NewMap();
	initData.config.DEBUG = false;
  	initData.config_desc("DEBUG") = "Set DEBUG flag to true in order to enable logging messages to the Opus Output Window.";

	initData.min_version = "12.0";
}

function OnAddCommands(addCmdData)
{
	var cmd = addCmdData.AddCommand();
	cmd.name = "ManageDopusFolderAliases";
	cmd.method = "OnManageDopusFolderAliases";
	cmd.desc = "Manage Dopus folder alaises";
	cmd.label = "ManageDopusFolderAliases";
	cmd.template = ""; 
}

//Custom commands

// Implement the ExternalCompare command
function OnManageDopusFolderAliases(scriptCmdData)
{
 	var dlg = DOpus.Dlg;
    dlg.window = scriptCmdData.func.sourcetab;
    dlg.template = "ManageDopusFolderAliases";
    dlg.detach = true;
    dlg.Show();
 	onDialogPopulateColumnsList(dlg);

    var closeDialog;
    while (!closeDialog) {
		var msg = dlg.GetMsg();
		if (!msg.result) break;
		//LogMessage("Msg: " + msg.control + " value: " + msg.value + " event:" + msg.event);
		switch (msg.control) {
			case "aliasList" :
				onDialogAliasSelected(dlg, msg);
				break;
			case "buttonAdd" :
				onDialogAddColumn(dlg, msg);
				break;
			case "buttonDelete" :
				onDialogDeleteColumn(dlg, msg);
				break;
			case "buttonApply" :
				 onDialogApplyChanges(dlg, msg);
				break;
			case "checkShowSystem" :
				 onDialogCheckShowSystem(dlg, msg);
				break;
			case "checkShowDead" :
				 onDialogShowDead(dlg, msg);
				break;
			case "buttonOK" :
				//params = DialogClosed(dlg, msg);
				closeDialog = true;
				break;
		} 
    }
    //LogMessage("Dialog Return code = " + dlg.result);
}

function forEach(enumerable, delegate)
{
    for (var enumerator = new Enumerator(enumerable); !enumerator.atEnd(); enumerator.moveNext())
    {
        delegate(enumerator.item());
    }
}

function onDialogPopulateColumnsList(dlg)
{
    var columnsList = dlg.Control("aliasList");
	columnsList.RemoveItem(-1);
	var selectedColumn;
 	
 	var includeSystemAliases = dlg.Control("checkShowSystem").value;
	var verifyAliases = dlg.Control("checkShowDead").value;

 	forEach(DOpus.aliases,
        function(alias)
        {
	  		//LogMessage("item: " + alias);
	    	if(alias)
	    	{
	    		if(includeSystemAliases || !alias.system)
	    		{
	    			if(!verifyAliases || !DOpus.FSUtil.Exists(alias.path))
	    			{
						var location = columnsList.AddItem(alias);
						var item = columnsList.GetItemAt(location);
						if(!selectedColumn)
						{ 
							item.selected = true;
							selectedColumn = alias;
							onDialogAliasSelected(dlg);
						}
	    			}
	 				//LogMessage("item: " + item.index + " location: " + location + " alias:" + alias.name);
	    		}
	 		}
    	}
    );
}

function GetAlias(name)
{
	//LogMessage("GetAlias: name" + name);
    for (var enumerator = new Enumerator(DOpus.aliases); !enumerator.atEnd(); enumerator.moveNext())
    {
    	if(enumerator.item() == name)
    	{
    		//LogMessage("GetAlias: name" + name + " Found item");
    		return enumerator.item();
    	}
    }
}

function onDialogCheckShowSystem(dlg)
{
	if(dlg.Control("checkShowSystem").value)
	{
		dlg.Control("checkShowDead").enabled = false;
		dlg.Control("checkShowDead").value = false;
	}
	else
	{
		dlg.Control("checkShowDead").enabled = true;
	}

	onDialogPopulateColumnsList(dlg);
}

function onDialogShowDead(dlg)
{
	//dlg.Control("checkShowDead").value;

	onDialogPopulateColumnsList(dlg);
}

// Dialog functions
function onDialogAliasSelected(dlg)
{
	var selectedAliasListItem = GetSelectedItem(dlg ,"aliasList");
	//LogMessage("onDialogAliasSelected: " + ((selectedAliasListItem && selectedAliasListItem.name) ? selectedAliasListItem.name : 'null'));
	if(!selectedAliasListItem) return;

	var selectedAlias = GetAlias(selectedAliasListItem.name);
    
    dlg.Control("aliasName").value = selectedAlias ? selectedAlias : "";
	dlg.Control("aliasPath").value = selectedAlias ? selectedAlias.path : "";
	dlg.Control("systemAliasCheckbox").value = selectedAlias ? selectedAlias.system : false;

	dlg.Control("buttonApply").enabled = selectedAlias ? !selectedAlias.system : false;
	dlg.Control("buttonDelete").enabled = selectedAlias ? !selectedAlias.system : false;
}

function onDialogApplyChanges(dlg, msg)
{
	var selectedAliasListItem = GetSelectedItem(dlg ,"aliasList");
	if(!selectedAliasListItem) return;

	var selectedName = selectedAliasListItem.name;
	var name = dlg.Control("aliasName").value;

    if(selectedName == name)
    {
    	var path = dlg.Control("aliasPath").value;
		DOpus.aliases.Add(name, path);
		//DOpus.aliases.Update();

    	//var selectedAlias = GetAlias(name);
    	//if(!(typeof(selectedAlias) == "undefined"))
		//{
		//	selectedAlias.path = path;
		//	//DOpus.aliases.Update();
		//}
    }
}

function onDialogAddColumn(dlg, msg)
{
  	var name = dlg.Control("aliasName").value;
	LogMessage("onDialogAddColumn: name:" + name);
  	var selectedAlias = GetAlias(name);

	if((typeof(selectedAlias) == "undefined"))
	{
		var path = dlg.Control("aliasPath").value;
		DOpus.aliases.Add(name, path);
		//DOpus.aliases.Update();
    	
		//customColumnsConfig.columns[customColumnsConfig.columns.length - 1]
		AddToAliasList(dlg, name, true);
		onDialogAliasSelected(dlg);
	}
}

function onDialogDeleteColumn(dlg, msg)
{
	var selectedAliasListItem = GetSelectedItem(dlg ,"aliasList");
	
	LogMessage("onDialogDeleteColumn: name:" + selectedAliasListItem.name);
  	var selectedAlias = GetAlias(selectedAliasListItem.name);

	if(!(typeof(selectedAlias) == "undefined") && !selectedAlias.system)
	{
		DOpus.aliases.Delete(selectedAliasListItem.name);
		//DOpus.aliases.Update();
    
    	RemoveFromAliasList(dlg, selectedAliasListItem);
		onDialogAliasSelected(dlg);
	}
}

function RemoveFromAliasList(dlg, listItem)
{
	var aliasList = dlg.Control("aliasList");
	var index = listItem.index;
	LogMessage("onDialogDeleteColumn: name:" + listItem.name + " index: " + index);
	aliasList.RemoveItem(listItem);

	if(aliasList.count > 0)
	{
		var previousListItem = aliasList.GetItemAt((index > 0) ? index -1 : 0);
    	previousListItem.selected = true;
	}
}

function AddToAliasList(dlg, name, setSelected)
{
	var aliasList = dlg.Control("aliasList");
 	var location = aliasList.AddItem(name);
 	if(setSelected)
 	{
		var item = aliasList.GetItemAt(location);
		item.selected = true;
	}
}

function GetSelectedItem(dlg, controlName)
{
	var listItems = dlg.Control(controlName);
    for(var i = 0; i < listItems.count; ++i) {
    	var column = listItems.GetItemAt(i);
	
		if(column && column.selected)
    	{
			return column;
 		}
    }
}

function SetSelectedItem(dlg, controlName, value)
{
	var listItems = dlg.Control(controlName);

	if(typeof(value) == "undefined") 
	{
		listItems.DeselectItem(-1);
		return;
	}
	var found = false;
	//listItems.GetItemByName(value);
    for(var i = 0; i < listItems.count; ++i) {
    	var item = listItems.GetItemAt(i);
	
		if(item && item.name == value)
    	{
    		LogMessage("SetSelectedItem: " + value + " " + found);
			found = true;
			item.selected = true;
			break;
 		}
    }
	if(!found) 
	{	LogMessage("SetSelectedItem: " + found);

		listItems.DeselectItem(-1);
	}
}

var doLogCmd = DOpus.NewCommand;

function IsDebugEnabled()
{
	return gbUseLogging || Script.config.DEBUG || doLogCmd.IsSet(Script.config.DebugEnableVar);
}

function LogMessage(message, force)
{
  	if (force || IsDebugEnabled()) { DOpus.OutputString(message)};
}

function InArray(col,arr){
	for (var i=0;i<arr.length;i++){
	if (col==arr[i].name) return true;
	}
}

==SCRIPT RESOURCES
<resources>
	<resource name="ManageDopusFolderAliases" type="dialog">
		<dialog fontsize="8" height="198" lang="english" resize="yes" title="Dopus Folder Alias management" width="366">
			<control close="0" height="14" name="buttonClose" resize="xy" title="Close" type="button" width="50" x="300" y="174" />
			<control height="186" name="aliasGroup" resize="h" title="Aliases" type="group" width="150" x="6" y="6" />
			<control height="126" name="aliasList" resize="h" type="listbox" width="138" x="12" y="42" />
			<control halign="left" height="8" name="aliasNameSatic" title="Alias Name" type="static" width="66" x="162" y="18" />
			<control halign="left" height="12" name="aliasName" resize="w" tip="Not set" type="edit" width="198" x="162" y="30" />
			<control halign="left" height="8" name="aliasPathStatic" title="Alias Path" type="static" width="60" x="162" y="48" />
			<control halign="left" height="12" name="aliasPath" resize="w" type="edit" width="198" x="162" y="60" />
			<control enable="no" height="10" name="systemAliasCheckbox" title="Is System Alias" type="check" width="64" x="162" y="78" />
			<control header="yes" height="64" name="group1" resize="yw" type="group" width="204" x="156" y="162" />
			<control default="yes" height="14" name="buttonApply" resize="xy" title="Save Changes" type="button" width="74" x="282" y="144" />
			<control height="14" name="buttonDelete" resize="y" title="Delete Alias" type="button" width="60" x="90" y="174" />
			<control height="14" name="buttonAdd" resize="y" title="Add Alias" type="button" width="60" x="12" y="174" />
			<control checked="yes" height="10" name="checkShowSystem" title="Include System Alises" type="check" width="126" x="18" y="18" />
			<control enable="no" height="10" name="checkShowDead" title="Show only dead paths" type="check" width="126" x="18" y="30" />
		</dialog>
	</resource>
</resources>
